@@ -373,7 +373,6 @@ image_create(struct display *display, const char *filename,
 	image->image = load_cairo_surface(filename);
 
 	if (!image->image) {
-		fprintf(stderr, "could not find the image %s!\n", b);
 		free(image->filename);
 		free(image);
 		return NULL;
@@ -412,6 +411,11 @@ main(int argc, char *argv[])
 	int i;
 	int image_counter = 0;
 
+	if (argc <= 1 || argv[1][0]=='-') {
+		printf("Usage: %s image...\n", argv[0]);
+		return 1;
+	}
+
 	d = display_create(&argc, argv);
 	if (d == NULL) {
 		fprintf(stderr, "failed to create display: %m\n");
@@ -424,5 +428,7 @@ main(int argc, char *argv[])
 	if (image_counter > 0)
 		display_run(d);
 
+	display_destroy(d);
+
 	return 0;
 }
