@@ -21,6 +21,8 @@
  * OF THIS SOFTWARE.
  */
 
+#include <config.h>
+
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
@@ -34,14 +36,18 @@
 
 #define ARRAY_LENGTH(a) (sizeof (a) / sizeof (a)[0])
 
+struct seat {
+	struct touch *touch;
+	struct wl_seat *seat;
+	struct wl_touch *wl_touch;
+};
+
 struct touch {
 	struct wl_display *display;
 	struct wl_registry *registry;
 	struct wl_compositor *compositor;
 	struct wl_shell *shell;
 	struct wl_shm *shm;
-	struct wl_seat *seat;
-	struct wl_touch *wl_touch;
 	struct wl_pointer *pointer;
 	struct wl_keyboard *keyboard;
 	struct wl_surface *surface;
@@ -95,7 +101,7 @@ shm_format(void *data, struct wl_shm *wl_shm, uint32_t format)
 		touch->has_argb = 1;
 }
 
-struct wl_shm_listener shm_listenter = {
+struct wl_shm_listener shm_listener = {
 	shm_format
 };
 
@@ -141,6 +147,7 @@ touch_paint(struct touch *touch, int32_t x, int32_t y, int32_t id)
 	p += touch->width;
 	p[2] = c;
 
+	wl_surface_attach(touch->surface, touch->buffer, 0, 0);
 	wl_surface_damage(touch->surface, x - 2, y - 2, 5, 5);
 	/* todo: We could queue up more damage before committing, if there
 	 * are more input events to handle.
@@ -196,18 +203,19 @@ static const struct wl_touch_listener touch_listener = {
 };
 
 static void
-seat_handle_capabilities(void *data, struct wl_seat *seat,
+seat_handle_capabilities(void *data, struct wl_seat *wl_seat,
 			 enum wl_seat_capability caps)
 {
-	struct touch *touch = data;
+	struct seat *seat = data;
+	struct touch *touch = seat->touch;
 
-	if ((caps & WL_SEAT_CAPABILITY_TOUCH) && !touch->wl_touch) {
-		touch->wl_touch = wl_seat_get_touch(seat);
-		wl_touch_set_user_data(touch->wl_touch, touch);
-		wl_touch_add_listener(touch->wl_touch, &touch_listener, touch);
-	} else if (!(caps & WL_SEAT_CAPABILITY_TOUCH) && touch->wl_touch) {
-		wl_touch_destroy(touch->wl_touch);
-		touch->wl_touch = NULL;
+	if ((caps & WL_SEAT_CAPABILITY_TOUCH) && !seat->wl_touch) {
+		seat->wl_touch = wl_seat_get_touch(wl_seat);
+		wl_touch_set_user_data(seat->wl_touch, touch);
+		wl_touch_add_listener(seat->wl_touch, &touch_listener, touch);
+	} else if (!(caps & WL_SEAT_CAPABILITY_TOUCH) && seat->wl_touch) {
+		wl_touch_destroy(seat->wl_touch);
+		seat->wl_touch = NULL;
 	}
 }
 
@@ -215,6 +223,21 @@ static const struct wl_seat_listener seat_listener = {
 	seat_handle_capabilities,
 };
 
+static void
+add_seat(struct touch *touch, uint32_t name, uint32_t version)
+{
+	struct seat *seat;
+
+	seat = malloc(sizeof *seat);
+	assert(seat);
+
+	seat->touch = touch;
+	seat->wl_touch = NULL;
+	seat->seat = wl_registry_bind(touch->registry, name,
+				      &wl_seat_interface, 1);
+	wl_seat_add_listener(seat->seat, &seat_listener, seat);
+}
+
 static void
 handle_ping(void *data, struct wl_shell_surface *shell_surface,
 	    uint32_t serial)
@@ -256,16 +279,20 @@ handle_global(void *data, struct wl_registry *registry,
 	} else if (strcmp(interface, "wl_shm") == 0) {
 		touch->shm = wl_registry_bind(registry, name,
 					      &wl_shm_interface, 1);
-		wl_shm_add_listener(touch->shm, &shm_listenter, touch);
+		wl_shm_add_listener(touch->shm, &shm_listener, touch);
 	} else if (strcmp(interface, "wl_seat") == 0) {
-		touch->seat = wl_registry_bind(registry, name,
-					       &wl_seat_interface, 1);
-		wl_seat_add_listener(touch->seat, &seat_listener, touch);
+		add_seat(touch, name, version);
 	}
 }
 
+static void
+handle_global_remove(void *data, struct wl_registry *registry, uint32_t name)
+{
+}
+
 static const struct wl_registry_listener registry_listener = {
-	handle_global
+	handle_global,
+	handle_global_remove
 };
 
 static struct touch *
@@ -274,6 +301,10 @@ touch_create(int width, int height)
 	struct touch *touch;
 
 	touch = malloc(sizeof *touch);
+	if (touch == NULL) {
+		fprintf(stderr, "out of memory\n");
+		exit(1);
+	}
 	touch->display = wl_display_connect(NULL);
 	assert(touch->display);
 
@@ -288,8 +319,6 @@ touch_create(int width, int height)
 		exit(1);
 	}
 
-	wl_display_get_fd(touch->display);
-	
 	touch->width = width;
 	touch->height = height;
 	touch->surface = wl_compositor_create_surface(touch->compositor);
