@@ -1,5 +1,5 @@
 /*
- * Copyright © 2011 Intel Corporation
+ * Copyright © 2011-2012 Intel Corporation
  *
  * Permission to use, copy, modify, distribute, and sell this software and
  * its documentation for any purpose is hereby granted without fee, provided
@@ -23,6 +23,7 @@
 #include "config.h"
 
 #include <stdlib.h>
+#include <linux/input.h>
 
 #include "compositor.h"
 
@@ -206,7 +207,8 @@ binding_key(struct weston_keyboard_grab *grab,
 			/* Don't send the key press event for the binding key */
 			return;
 		}
-	} else if (!wl_list_empty(&keyboard->focus_resource_list)) {
+	}
+	if (!wl_list_empty(&keyboard->focus_resource_list)) {
 		serial = wl_display_next_serial(display);
 		wl_resource_for_each(resource, &keyboard->focus_resource_list) {
 			wl_keyboard_send_key(resource,
@@ -248,7 +250,8 @@ static const struct weston_keyboard_grab_interface binding_grab = {
 };
 
 static void
-install_binding_grab(struct weston_seat *seat, uint32_t time, uint32_t key)
+install_binding_grab(struct weston_seat *seat, uint32_t time, uint32_t key,
+                     struct weston_surface *focus)
 {
 	struct binding_keyboard_grab *grab;
 
@@ -256,6 +259,19 @@ install_binding_grab(struct weston_seat *seat, uint32_t time, uint32_t key)
 	grab->key = key;
 	grab->grab.interface = &binding_grab;
 	weston_keyboard_start_grab(seat->keyboard, &grab->grab);
+
+	/* Notify the surface which had the focus before this binding
+	 * triggered that we stole a keypress from under it, by forcing
+	 * a wl_keyboard leave/enter pair. The enter event will contain
+	 * the pressed key in the keys array, so the client will know
+	 * the exact state of the keyboard.
+	 * If the old focus surface is different than the new one it
+	 * means it was changed in the binding handler, so it received
+	 * the enter event already. */
+	if (focus && seat->keyboard->focus == focus) {
+		weston_keyboard_set_focus(seat->keyboard, NULL);
+		weston_keyboard_set_focus(seat->keyboard, focus);
+	}
 }
 
 WL_EXPORT void
@@ -264,7 +280,8 @@ weston_compositor_run_key_binding(struct weston_compositor *compositor,
 				  uint32_t time, uint32_t key,
 				  enum wl_keyboard_key_state state)
 {
-	struct weston_binding *b;
+	struct weston_binding *b, *tmp;
+	struct weston_surface *focus;
 
 	if (state == WL_KEYBOARD_KEY_STATE_RELEASED)
 		return;
@@ -273,9 +290,10 @@ weston_compositor_run_key_binding(struct weston_compositor *compositor,
 	wl_list_for_each(b, &compositor->modifier_binding_list, link)
 		b->key = key;
 
-	wl_list_for_each(b, &compositor->key_binding_list, link) {
+	wl_list_for_each_safe(b, tmp, &compositor->key_binding_list, link) {
 		if (b->key == key && b->modifier == seat->modifier_state) {
 			weston_key_binding_handler_t handler = b->handler;
+			focus = seat->keyboard->focus;
 			handler(seat, time, key, b->data);
 
 			/* If this was a key binding and it didn't
@@ -283,7 +301,7 @@ weston_compositor_run_key_binding(struct weston_compositor *compositor,
 			 * swallow the key press. */
 			if (seat->keyboard->grab ==
 			    &seat->keyboard->default_grab)
-				install_binding_grab(seat, time, key);
+				install_binding_grab(seat, time, key, focus);
 		}
 	}
 }
@@ -294,12 +312,12 @@ weston_compositor_run_modifier_binding(struct weston_compositor *compositor,
 				       enum weston_keyboard_modifier modifier,
 				       enum wl_keyboard_key_state state)
 {
-	struct weston_binding *b;
+	struct weston_binding *b, *tmp;
 
 	if (seat->keyboard->grab != &seat->keyboard->default_grab)
 		return;
 
-	wl_list_for_each(b, &compositor->modifier_binding_list, link) {
+	wl_list_for_each_safe(b, tmp, &compositor->modifier_binding_list, link) {
 		weston_modifier_binding_handler_t handler = b->handler;
 
 		if (b->modifier != modifier)
@@ -325,7 +343,7 @@ weston_compositor_run_button_binding(struct weston_compositor *compositor,
 				     uint32_t time, uint32_t button,
 				     enum wl_pointer_button_state state)
 {
-	struct weston_binding *b;
+	struct weston_binding *b, *tmp;
 
 	if (state == WL_POINTER_BUTTON_STATE_RELEASED)
 		return;
@@ -334,7 +352,7 @@ weston_compositor_run_button_binding(struct weston_compositor *compositor,
 	wl_list_for_each(b, &compositor->modifier_binding_list, link)
 		b->key = button;
 
-	wl_list_for_each(b, &compositor->button_binding_list, link) {
+	wl_list_for_each_safe(b, tmp, &compositor->button_binding_list, link) {
 		if (b->button == button && b->modifier == seat->modifier_state) {
 			weston_button_binding_handler_t handler = b->handler;
 			handler(seat, time, button, b->data);
@@ -347,12 +365,12 @@ weston_compositor_run_touch_binding(struct weston_compositor *compositor,
 				    struct weston_seat *seat, uint32_t time,
 				    int touch_type)
 {
-	struct weston_binding *b;
+	struct weston_binding *b, *tmp;
 
 	if (seat->touch->num_tp != 1 || touch_type != WL_TOUCH_DOWN)
 		return;
 
-	wl_list_for_each(b, &compositor->touch_binding_list, link) {
+	wl_list_for_each_safe(b, tmp, &compositor->touch_binding_list, link) {
 		if (b->modifier == seat->modifier_state) {
 			weston_touch_binding_handler_t handler = b->handler;
 			handler(seat, time, b->data);
@@ -366,13 +384,13 @@ weston_compositor_run_axis_binding(struct weston_compositor *compositor,
 				   uint32_t time, uint32_t axis,
 				   wl_fixed_t value)
 {
-	struct weston_binding *b;
+	struct weston_binding *b, *tmp;
 
 	/* Invalidate all active modifier bindings. */
 	wl_list_for_each(b, &compositor->modifier_binding_list, link)
 		b->key = axis;
 
-	wl_list_for_each(b, &compositor->axis_binding_list, link) {
+	wl_list_for_each_safe(b, tmp, &compositor->axis_binding_list, link) {
 		if (b->axis == axis && b->modifier == seat->modifier_state) {
 			weston_axis_binding_handler_t handler = b->handler;
 			handler(seat, time, axis, value, b->data);
@@ -390,10 +408,10 @@ weston_compositor_run_debug_binding(struct weston_compositor *compositor,
 				    enum wl_keyboard_key_state state)
 {
 	weston_key_binding_handler_t handler;
-	struct weston_binding *binding;
+	struct weston_binding *binding, *tmp;
 	int count = 0;
 
-	wl_list_for_each(binding, &compositor->debug_binding_list, link) {
+	wl_list_for_each_safe(binding, tmp, &compositor->debug_binding_list, link) {
 		if (key != binding->key)
 			continue;
 
@@ -404,3 +422,147 @@ weston_compositor_run_debug_binding(struct weston_compositor *compositor,
 
 	return count;
 }
+
+struct debug_binding_grab {
+	struct weston_keyboard_grab grab;
+	struct weston_seat *seat;
+	uint32_t key[2];
+	int key_released[2];
+};
+
+static void
+debug_binding_key(struct weston_keyboard_grab *grab, uint32_t time,
+		  uint32_t key, uint32_t state)
+{
+	struct debug_binding_grab *db = (struct debug_binding_grab *) grab;
+	struct weston_compositor *ec = db->seat->compositor;
+	struct wl_display *display = ec->wl_display;
+	struct wl_resource *resource;
+	uint32_t serial;
+	int send = 0, terminate = 0;
+	int check_binding = 1;
+	int i;
+	struct wl_list *resource_list;
+
+	if (state == WL_KEYBOARD_KEY_STATE_RELEASED) {
+		/* Do not run bindings on key releases */
+		check_binding = 0;
+
+		for (i = 0; i < 2; i++)
+			if (key == db->key[i])
+				db->key_released[i] = 1;
+
+		if (db->key_released[0] && db->key_released[1]) {
+			/* All key releases been swalled so end the grab */
+			terminate = 1;
+		} else if (key != db->key[0] && key != db->key[1]) {
+			/* Should not swallow release of other keys */
+			send = 1;
+		}
+	} else if (key == db->key[0] && !db->key_released[0]) {
+		/* Do not check bindings for the first press of the binding
+		 * key. This allows it to be used as a debug shortcut.
+		 * We still need to swallow this event. */
+		check_binding = 0;
+	} else if (db->key[1]) {
+		/* If we already ran a binding don't process another one since
+		 * we can't keep track of all the binding keys that were
+		 * pressed in order to swallow the release events. */
+		send = 1;
+		check_binding = 0;
+	}
+
+	if (check_binding) {
+		if (weston_compositor_run_debug_binding(ec, db->seat, time,
+							key, state)) {
+			/* We ran a binding so swallow the press and keep the
+			 * grab to swallow the released too. */
+			send = 0;
+			terminate = 0;
+			db->key[1] = key;
+		} else {
+			/* Terminate the grab since the key pressed is not a
+			 * debug binding key. */
+			send = 1;
+			terminate = 1;
+		}
+	}
+
+	if (send) {
+		serial = wl_display_next_serial(display);
+		resource_list = &grab->keyboard->focus_resource_list;
+		wl_resource_for_each(resource, resource_list) {
+			wl_keyboard_send_key(resource, serial, time, key, state);
+		}
+	}
+
+	if (terminate) {
+		weston_keyboard_end_grab(grab->keyboard);
+		if (grab->keyboard->input_method_resource)
+			grab->keyboard->grab = &grab->keyboard->input_method_grab;
+		free(db);
+	}
+}
+
+static void
+debug_binding_modifiers(struct weston_keyboard_grab *grab, uint32_t serial,
+			uint32_t mods_depressed, uint32_t mods_latched,
+			uint32_t mods_locked, uint32_t group)
+{
+	struct wl_resource *resource;
+	struct wl_list *resource_list;
+
+	resource_list = &grab->keyboard->focus_resource_list;
+
+	wl_resource_for_each(resource, resource_list) {
+		wl_keyboard_send_modifiers(resource, serial, mods_depressed,
+					   mods_latched, mods_locked, group);
+	}
+}
+
+static void
+debug_binding_cancel(struct weston_keyboard_grab *grab)
+{
+	struct debug_binding_grab *db = (struct debug_binding_grab *) grab;
+
+	weston_keyboard_end_grab(grab->keyboard);
+	free(db);
+}
+
+struct weston_keyboard_grab_interface debug_binding_keyboard_grab = {
+	debug_binding_key,
+	debug_binding_modifiers,
+	debug_binding_cancel,
+};
+
+static void
+debug_binding(struct weston_seat *seat, uint32_t time, uint32_t key, void *data)
+{
+	struct debug_binding_grab *grab;
+
+	grab = calloc(1, sizeof *grab);
+	if (!grab)
+		return;
+
+	grab->seat = seat;
+	grab->key[0] = key;
+	grab->grab.interface = &debug_binding_keyboard_grab;
+	weston_keyboard_start_grab(seat->keyboard, &grab->grab);
+}
+
+/** Install the trigger binding for debug bindings.
+ *
+ * \param compositor The compositor.
+ * \param mod The modifier.
+ *
+ * This will add a key binding for modifier+SHIFT+SPACE that will trigger
+ * debug key bindings.
+ */
+WL_EXPORT void
+weston_install_debug_key_binding(struct weston_compositor *compositor,
+				 uint32_t mod)
+{
+	weston_compositor_add_key_binding(compositor, KEY_SPACE,
+					  mod | MODIFIER_SHIFT,
+					  debug_binding, NULL);
+}
