@@ -42,8 +42,9 @@
 #include "compositor.h"
 #include "launcher-util.h"
 #include "pixman-renderer.h"
-#include "udev-input.h"
+#include "libinput-seat.h"
 #include "gl-renderer.h"
+#include "presentation_timing-server-protocol.h"
 
 struct fbdev_compositor {
 	struct weston_compositor base;
@@ -114,12 +115,10 @@ to_fbdev_compositor(struct weston_compositor *base)
 static void
 fbdev_output_start_repaint_loop(struct weston_output *output)
 {
-	uint32_t msec;
-	struct timeval tv;
+	struct timespec ts;
 
-	gettimeofday(&tv, NULL);
-	msec = tv.tv_sec * 1000 + tv.tv_usec / 1000;
-	weston_output_finish_frame(output, msec);
+	weston_compositor_read_presentation_clock(output->compositor, &ts);
+	weston_output_finish_frame(output, &ts, PRESENTATION_FEEDBACK_INVALID);
 }
 
 static void
@@ -128,57 +127,36 @@ fbdev_output_repaint_pixman(struct weston_output *base, pixman_region32_t *damag
 	struct fbdev_output *output = to_fbdev_output(base);
 	struct weston_compositor *ec = output->base.compositor;
 	pixman_box32_t *rects;
-	int nrects, i, src_x, src_y, x1, y1, x2, y2, width, height;
+	int nrects, i;
 
 	/* Repaint the damaged region onto the back buffer. */
 	pixman_renderer_output_set_buffer(base, output->shadow_surface);
 	ec->renderer->repaint_output(base, damage);
 
 	/* Transform and composite onto the frame buffer. */
-	width = pixman_image_get_width(output->shadow_surface);
-	height = pixman_image_get_height(output->shadow_surface);
 	rects = pixman_region32_rectangles(damage, &nrects);
 
 	for (i = 0; i < nrects; i++) {
-		switch (base->transform) {
-		default:
-		case WL_OUTPUT_TRANSFORM_NORMAL:
-			x1 = rects[i].x1;
-			x2 = rects[i].x2;
-			y1 = rects[i].y1;
-			y2 = rects[i].y2;
-			break;
-		case WL_OUTPUT_TRANSFORM_180:
-			x1 = width - rects[i].x2;
-			x2 = width - rects[i].x1;
-			y1 = height - rects[i].y2;
-			y2 = height - rects[i].y1;
-			break;
-		case WL_OUTPUT_TRANSFORM_90:
-			x1 = height - rects[i].y2;
-			x2 = height - rects[i].y1;
-			y1 = rects[i].x1;
-			y2 = rects[i].x2;
-			break;
-		case WL_OUTPUT_TRANSFORM_270:
-			x1 = rects[i].y1;
-			x2 = rects[i].y2;
-			y1 = width - rects[i].x2;
-			y2 = width - rects[i].x1;
-			break;
-		}
-		src_x = x1;
-		src_y = y1;
+		pixman_box32_t transformed_rect;
+		int width, height;
 
+		transformed_rect = weston_transformed_rect(base->width,
+							   base->height,
+							   base->transform,
+							   1, rects[i]);
+		width = transformed_rect.x2 - transformed_rect.x1;
+		height = transformed_rect.y2 - transformed_rect.y1;
 		pixman_image_composite32(PIXMAN_OP_SRC,
 			output->shadow_surface, /* src */
 			NULL /* mask */,
 			output->hw_surface, /* dest */
-			src_x, src_y, /* src_x, src_y */
+			transformed_rect.x1, /* src_x */
+			transformed_rect.y1, /* src_y */
 			0, 0, /* mask_x, mask_y */
-			x1, y1, /* dest_x, dest_y */
-			x2 - x1, /* width */
-			y2 - y1 /* height */);
+			transformed_rect.x1, /* dest_x */
+			transformed_rect.y1, /* dest_y */
+			width, /* width */
+			height /* height */);
 	}
 
 	/* Update the damage region. */
@@ -222,8 +200,10 @@ static int
 finish_frame_handler(void *data)
 {
 	struct fbdev_output *output = data;
+	struct timespec ts;
 
-	fbdev_output_start_repaint_loop(&output->base);
+	weston_compositor_read_presentation_clock(output->base.compositor, &ts);
+	weston_output_finish_frame(&output->base, &ts, 0);
 
 	return 1;
 }
@@ -501,17 +481,18 @@ fbdev_output_create(struct fbdev_compositor *compositor,
                     const char *device)
 {
 	struct fbdev_output *output;
-	pixman_transform_t transform;
+	struct weston_config_section *section;
 	int fb_fd;
-	int shadow_width, shadow_height;
 	int width, height;
 	unsigned int bytes_per_pixel;
 	struct wl_event_loop *loop;
+	uint32_t config_transform;
+	char *s;
 
 	weston_log("Creating fbdev output.\n");
 
-	output = calloc(1, sizeof *output);
-	if (!output)
+	output = zalloc(sizeof *output);
+	if (output == NULL)
 		return -1;
 
 	output->compositor = compositor;
@@ -535,10 +516,6 @@ fbdev_output_create(struct fbdev_compositor *compositor,
 	output->base.start_repaint_loop = fbdev_output_start_repaint_loop;
 	output->base.repaint = fbdev_output_repaint;
 	output->base.destroy = fbdev_output_destroy;
-	output->base.assign_planes = NULL;
-	output->base.set_backlight = NULL;
-	output->base.set_dpms = NULL;
-	output->base.switch_mode = NULL;
 
 	/* only one static mode in list */
 	output->mode.flags =
@@ -553,84 +530,47 @@ fbdev_output_create(struct fbdev_compositor *compositor,
 	output->base.subpixel = WL_OUTPUT_SUBPIXEL_UNKNOWN;
 	output->base.make = "unknown";
 	output->base.model = output->fb_info.id;
+	output->base.name = strdup("fbdev");
+
+	section = weston_config_get_section(compositor->base.config,
+					    "output", "name",
+					    output->base.name);
+	weston_config_section_get_string(section, "transform", &s, "normal");
+	if (weston_parse_transform(s, &config_transform) < 0)
+		weston_log("Invalid transform \"%s\" for output %s\n",
+			   s, output->base.name);
+	free(s);
 
 	weston_output_init(&output->base, &compositor->base,
 	                   0, 0, output->fb_info.width_mm,
 	                   output->fb_info.height_mm,
-	                   WL_OUTPUT_TRANSFORM_NORMAL,
+	                   config_transform,
 			   1);
 
-	width = output->fb_info.x_resolution;
-	height = output->fb_info.y_resolution;
-
-	pixman_transform_init_identity(&transform);
-	switch (output->base.transform) {
-	default:
-	case WL_OUTPUT_TRANSFORM_NORMAL:
-		shadow_width = width;
-		shadow_height = height;
-		pixman_transform_rotate(&transform,
-			NULL, 0, 0);
-		pixman_transform_translate(&transform, NULL,
-			0, 0);
-		break;
-	case WL_OUTPUT_TRANSFORM_180:
-		shadow_width = width;
-		shadow_height = height;
-		pixman_transform_rotate(&transform,
-			NULL, -pixman_fixed_1, 0);
-		pixman_transform_translate(NULL, &transform,
-			pixman_int_to_fixed(shadow_width),
-			pixman_int_to_fixed(shadow_height));
-		break;
-	case WL_OUTPUT_TRANSFORM_270:
-		shadow_width = height;
-		shadow_height = width;
-		pixman_transform_rotate(&transform,
-			NULL, 0, pixman_fixed_1);
-		pixman_transform_translate(&transform,
-			NULL,
-			pixman_int_to_fixed(shadow_width),
-			0);
-		break;
-	case WL_OUTPUT_TRANSFORM_90:
-		shadow_width = height;
-		shadow_height = width;
-		pixman_transform_rotate(&transform,
-			NULL, 0, -pixman_fixed_1);
-		pixman_transform_translate(&transform,
-			NULL,
-			0,
-			pixman_int_to_fixed(shadow_height));
-		break;
-	}
-
+	width = output->mode.width;
+	height = output->mode.height;
 	bytes_per_pixel = output->fb_info.bits_per_pixel / 8;
 
 	output->shadow_buf = malloc(width * height * bytes_per_pixel);
 	output->shadow_surface =
 		pixman_image_create_bits(output->fb_info.pixel_format,
-		                         shadow_width, shadow_height,
+		                         width, height,
 		                         output->shadow_buf,
-		                         shadow_width * bytes_per_pixel);
+		                         width * bytes_per_pixel);
 	if (output->shadow_buf == NULL || output->shadow_surface == NULL) {
 		weston_log("Failed to create surface for frame buffer.\n");
 		goto out_hw_surface;
 	}
 
-	/* No need in transform for normal output */
-	if (output->base.transform != WL_OUTPUT_TRANSFORM_NORMAL)
-		pixman_image_set_transform(output->shadow_surface, &transform);
-
 	if (compositor->use_pixman) {
 		if (pixman_renderer_output_create(&output->base) < 0)
 			goto out_shadow_surface;
 	} else {
 		setenv("HYBRIS_EGLPLATFORM", "wayland", 1);
 		if (gl_renderer->output_create(&output->base,
-					       (EGLNativeWindowType)NULL,
+					       (EGLNativeWindowType)NULL, NULL,
 					       gl_renderer->opaque_attribs,
-					       NULL) < 0) {
+					       NULL, 0) < 0) {
 			weston_log("gl_renderer_output_create failed.\n");
 			goto out_shadow_surface;
 		}
@@ -641,7 +581,7 @@ fbdev_output_create(struct fbdev_compositor *compositor,
 	output->finish_frame_timer =
 		wl_event_loop_add_timer(loop, finish_frame_handler, output);
 
-	wl_list_insert(compositor->base.output_list.prev, &output->base.link);
+	weston_compositor_add_output(&compositor->base, &output->base);
 
 	weston_log("fbdev output %d×%d px\n",
 	           output->mode.width, output->mode.height);
@@ -847,7 +787,7 @@ session_notify(struct wl_listener *listener, void *data)
 				 &compositor->base.output_list, link) {
 			output->repaint_needed = 0;
 		}
-	};
+	}
 }
 
 static void
@@ -875,7 +815,7 @@ fbdev_compositor_create(struct wl_display *display, int *argc, char *argv[],
 
 	weston_log("initializing fbdev backend\n");
 
-	compositor = calloc(1, sizeof *compositor);
+	compositor = zalloc(sizeof *compositor);
 	if (compositor == NULL)
 		return NULL;
 
@@ -883,6 +823,10 @@ fbdev_compositor_create(struct wl_display *display, int *argc, char *argv[],
 	                           config) < 0)
 		goto out_free;
 
+	if (weston_compositor_set_presentation_clock_software(
+							&compositor->base) < 0)
+		goto out_compositor;
+
 	compositor->udev = udev_new();
 	if (compositor->udev == NULL) {
 		weston_log("Failed to initialize udev context.\n");
@@ -893,8 +837,9 @@ fbdev_compositor_create(struct wl_display *display, int *argc, char *argv[],
 	compositor->session_listener.notify = session_notify;
 	wl_signal_add(&compositor->base.session_signal,
 		      &compositor->session_listener);
-	compositor->base.launcher =
-		weston_launcher_connect(&compositor->base, param->tty, "seat0");
+	compositor->base.launcher = weston_launcher_connect(&compositor->base,
+							    param->tty, "seat0",
+							    false);
 	if (!compositor->base.launcher) {
 		weston_log("fatal: fbdev backend should be run "
 			   "using weston-launch binary or as root\n");
@@ -923,9 +868,10 @@ fbdev_compositor_create(struct wl_display *display, int *argc, char *argv[],
 			goto out_launcher;
 		}
 
-		if (gl_renderer->create(&compositor->base, EGL_DEFAULT_DISPLAY,
+		if (gl_renderer->create(&compositor->base, NO_EGL_PLATFORM,
+					EGL_DEFAULT_DISPLAY,
 					gl_renderer->opaque_attribs,
-					NULL) < 0) {
+					NULL, 0) < 0) {
 			weston_log("gl_renderer_create failed.\n");
 			goto out_launcher;
 		}
