@@ -43,6 +43,7 @@
 #include "../shared/os-compatibility.h"
 #include "../shared/cairo-util.h"
 #include "fullscreen-shell-client-protocol.h"
+#include "presentation_timing-server-protocol.h"
 
 #define WINDOW_TITLE "Weston Compositor"
 
@@ -306,9 +307,21 @@ static void
 frame_done(void *data, struct wl_callback *callback, uint32_t time)
 {
 	struct weston_output *output = data;
+	struct timespec ts;
 
 	wl_callback_destroy(callback);
-	weston_output_finish_frame(output, time);
+
+	/* XXX: use the presentation extension for proper timings */
+
+	/*
+	 * This is the fallback case, where Presentation extension is not
+	 * available from the parent compositor. We do not know the base for
+	 * 'time', so we cannot feed it to finish_frame(). Do the only thing
+	 * we can, and pretend finish_frame time is when we process this
+	 * event.
+	 */
+	weston_compositor_read_presentation_clock(output->compositor, &ts);
+	weston_output_finish_frame(output, &ts, 0);
 }
 
 static const struct wl_callback_listener frame_listener = {
@@ -632,9 +645,11 @@ wayland_output_init_gl_renderer(struct wayland_output *output)
 	}
 
 	if (gl_renderer->output_create(&output->base,
+				       output->gl.egl_window,
 				       output->gl.egl_window,
 				       gl_renderer->alpha_attribs,
-				       NULL) < 0)
+				       NULL,
+				       0) < 0)
 		goto cleanup_window;
 
 	return 0;
@@ -1043,7 +1058,7 @@ wayland_output_create(struct wayland_compositor *c, int x, int y,
 	output->base.set_dpms = NULL;
 	output->base.switch_mode = wayland_output_switch_mode;
 
-	wl_list_insert(c->base.output_list.prev, &output->base.link);
+	weston_compositor_add_output(&c->base, &output->base);
 
 	return output;
 
@@ -1072,18 +1087,7 @@ wayland_output_create_for_config(struct wayland_compositor *c,
 	char *mode, *t, *name, *str;
 	int width, height, scale;
 	uint32_t transform;
-	unsigned int i, slen;
-
-	static const struct { const char *name; uint32_t token; } transform_names[] = {
-		{ "normal",	WL_OUTPUT_TRANSFORM_NORMAL },
-		{ "90",		WL_OUTPUT_TRANSFORM_90 },
-		{ "180",	WL_OUTPUT_TRANSFORM_180 },
-		{ "270",	WL_OUTPUT_TRANSFORM_270 },
-		{ "flipped",	WL_OUTPUT_TRANSFORM_FLIPPED },
-		{ "flipped-90",	WL_OUTPUT_TRANSFORM_FLIPPED_90 },
-		{ "flipped-180", WL_OUTPUT_TRANSFORM_FLIPPED_180 },
-		{ "flipped-270", WL_OUTPUT_TRANSFORM_FLIPPED_270 },
-	};
+	unsigned int slen;
 
 	weston_config_section_get_string(config_section, "name", &name, NULL);
 	if (name) {
@@ -1120,15 +1124,9 @@ wayland_output_create_for_config(struct wayland_compositor *c,
 
 	weston_config_section_get_string(config_section,
 					 "transform", &t, "normal");
-	transform = WL_OUTPUT_TRANSFORM_NORMAL;
-	for (i = 0; i < ARRAY_LENGTH(transform_names); i++) {
-		if (strcmp(transform_names[i].name, t) == 0) {
-			transform = transform_names[i].token;
-			break;
-		}
-	}
-	if (i >= ARRAY_LENGTH(transform_names))
-		weston_log("Invalid transform \"%s\" for output %s\n", t, name);
+	if (weston_parse_transform(t, &transform) < 0)
+		weston_log("Invalid transform \"%s\" for output %s\n",
+			   t, name);
 	free(t);
 
 	output = wayland_output_create(c, x, y, width, height, name, 0,
@@ -1309,6 +1307,9 @@ input_handle_pointer_leave(void *data, struct wl_pointer *pointer,
 {
 	struct wayland_input *input = data;
 
+	if (!input->output)
+		return;
+
 	if (input->output->frame) {
 		frame_pointer_leave(input->output->frame, input);
 
@@ -1329,6 +1330,9 @@ input_handle_motion(void *data, struct wl_pointer *pointer,
 	int32_t fx, fy;
 	enum theme_location location;
 
+	if (!input->output)
+		return;
+
 	if (input->output->frame) {
 		location = frame_pointer_motion(input->output->frame, input,
 						wl_fixed_to_int(x),
@@ -1370,6 +1374,9 @@ input_handle_button(void *data, struct wl_pointer *pointer,
 	enum frame_button_state fstate;
 	enum theme_location location;
 
+	if (!input->output)
+		return;
+
 	if (input->output->frame) {
 		fstate = state == WL_POINTER_BUTTON_STATE_PRESSED ?
 			FRAME_BUTTON_PRESSED : FRAME_BUTTON_RELEASED;
@@ -1386,8 +1393,16 @@ input_handle_button(void *data, struct wl_pointer *pointer,
 			return;
 		}
 
-		if (frame_status(input->output->frame) & FRAME_STATUS_CLOSE)
-			wl_display_terminate(input->compositor->base.wl_display);
+		if (frame_status(input->output->frame) & FRAME_STATUS_CLOSE) {
+			wayland_output_destroy(&input->output->base);
+			input->output = NULL;
+			input->keyboard_focus = NULL;
+
+			if (wl_list_empty(&input->compositor->base.output_list))
+				wl_display_terminate(input->compositor->base.wl_display);
+
+			return;
+		}
 
 		if (frame_status(input->output->frame) & FRAME_STATUS_REPAINT)
 			weston_output_schedule_repaint(&input->output->base);
@@ -1523,7 +1538,7 @@ input_handle_keyboard_leave(void *data,
 
 	focus = input->keyboard_focus;
 	if (!focus)
-		return; /* This shouldn't happen */
+		return;
 
 	focus->keyboard_count--;
 	if (!focus->keyboard_count && focus->frame) {
@@ -1943,8 +1958,10 @@ wayland_compositor_create(struct wl_display *display, int use_pixman,
 				   config) < 0)
 		goto err_free;
 
-	c->parent.wl_display = wl_display_connect(display_name);
+	if (weston_compositor_set_presentation_clock_software(&c->base) < 0)
+		goto err_compositor;
 
+	c->parent.wl_display = wl_display_connect(display_name);
 	if (c->parent.wl_display == NULL) {
 		weston_log("failed to create display: %m\n");
 		goto err_compositor;
@@ -1970,9 +1987,12 @@ wayland_compositor_create(struct wl_display *display, int use_pixman,
 	}
 
 	if (!c->use_pixman) {
-		if (gl_renderer->create(&c->base, c->parent.wl_display,
-				gl_renderer->alpha_attribs,
-				NULL) < 0) {
+		if (gl_renderer->create(&c->base,
+					EGL_PLATFORM_WAYLAND_KHR,
+					c->parent.wl_display,
+					gl_renderer->alpha_attribs,
+					NULL,
+					0) < 0) {
 			weston_log("Failed to initialize the GL renderer; "
 				   "falling back to pixman.\n");
 			c->use_pixman = 1;
