@@ -108,7 +108,7 @@ touch_focus_view_destroyed(struct wl_listener *listener, void *data)
 		container_of(listener, struct weston_touch,
 			     focus_view_listener);
 
-	weston_touch_set_focus(touch->seat, NULL);
+	weston_touch_set_focus(touch, NULL);
 }
 
 static void
@@ -118,7 +118,7 @@ touch_focus_resource_destroyed(struct wl_listener *listener, void *data)
 		container_of(listener, struct weston_touch,
 			     focus_resource_listener);
 
-	weston_touch_set_focus(touch->seat, NULL);
+	weston_touch_set_focus(touch, NULL);
 }
 
 static void
@@ -423,12 +423,15 @@ static const struct weston_keyboard_grab_interface
 static void
 pointer_unmap_sprite(struct weston_pointer *pointer)
 {
-	if (weston_surface_is_mapped(pointer->sprite->surface))
-		weston_surface_unmap(pointer->sprite->surface);
+	struct weston_surface *surface = pointer->sprite->surface;
+
+	if (weston_surface_is_mapped(surface))
+		weston_surface_unmap(surface);
 
 	wl_list_remove(&pointer->sprite_destroy_listener.link);
-	pointer->sprite->surface->configure = NULL;
-	pointer->sprite->surface->configure_private = NULL;
+	surface->configure = NULL;
+	surface->configure_private = NULL;
+	weston_surface_set_label_func(surface, NULL);
 	weston_view_destroy(pointer->sprite);
 	pointer->sprite = NULL;
 }
@@ -751,14 +754,13 @@ weston_keyboard_set_focus(struct weston_keyboard *keyboard,
 	wl_signal_emit(&keyboard->focus_signal, keyboard);
 }
 
+/* Users of this function must manually manage the keyboard focus */
 WL_EXPORT void
 weston_keyboard_start_grab(struct weston_keyboard *keyboard,
 			   struct weston_keyboard_grab *grab)
 {
 	keyboard->grab = grab;
 	grab->keyboard = keyboard;
-
-	/* XXX focus? */
 }
 
 WL_EXPORT void
@@ -1319,8 +1321,6 @@ notify_key(struct weston_seat *seat, uint32_t time, uint32_t key,
 
 	if (state == WL_KEYBOARD_KEY_STATE_PRESSED) {
 		weston_compositor_idle_inhibit(compositor);
-		keyboard->grab_key = key;
-		keyboard->grab_time = time;
 	} else {
 		weston_compositor_idle_release(compositor);
 	}
@@ -1359,6 +1359,13 @@ notify_key(struct weston_seat *seat, uint32_t time, uint32_t key,
 				      key,
 				      state);
 	}
+
+	if (state == WL_KEYBOARD_KEY_STATE_PRESSED) {
+		keyboard->grab_serial =
+			wl_display_get_serial(compositor->wl_display);
+		keyboard->grab_time = time;
+		keyboard->grab_key = key;
+	}
 }
 
 WL_EXPORT void
@@ -1442,25 +1449,25 @@ notify_keyboard_focus_out(struct weston_seat *seat)
 }
 
 WL_EXPORT void
-weston_touch_set_focus(struct weston_seat *seat, struct weston_view *view)
+weston_touch_set_focus(struct weston_touch *touch, struct weston_view *view)
 {
 	struct wl_list *focus_resource_list;
 
-	focus_resource_list = &seat->touch->focus_resource_list;
+	focus_resource_list = &touch->focus_resource_list;
 
-	if (view && seat->touch->focus &&
-	    seat->touch->focus->surface == view->surface) {
-		seat->touch->focus = view;
+	if (view && touch->focus &&
+	    touch->focus->surface == view->surface) {
+		touch->focus = view;
 		return;
 	}
 
-	wl_list_remove(&seat->touch->focus_resource_listener.link);
-	wl_list_init(&seat->touch->focus_resource_listener.link);
-	wl_list_remove(&seat->touch->focus_view_listener.link);
-	wl_list_init(&seat->touch->focus_view_listener.link);
+	wl_list_remove(&touch->focus_resource_listener.link);
+	wl_list_init(&touch->focus_resource_listener.link);
+	wl_list_remove(&touch->focus_view_listener.link);
+	wl_list_init(&touch->focus_view_listener.link);
 
 	if (!wl_list_empty(focus_resource_list)) {
-		move_resources(&seat->touch->resource_list,
+		move_resources(&touch->resource_list,
 			       focus_resource_list);
 	}
 
@@ -1468,19 +1475,19 @@ weston_touch_set_focus(struct weston_seat *seat, struct weston_view *view)
 		struct wl_client *surface_client;
 
 		if (!view->surface->resource) {
-			seat->touch->focus = NULL;
+			touch->focus = NULL;
 			return;
 		}
 
 		surface_client = wl_resource_get_client(view->surface->resource);
 		move_resources_for_client(focus_resource_list,
-					  &seat->touch->resource_list,
+					  &touch->resource_list,
 					  surface_client);
 		wl_resource_add_destroy_listener(view->surface->resource,
-						 &seat->touch->focus_resource_listener);
-		wl_signal_add(&view->destroy_signal, &seat->touch->focus_view_listener);
+						 &touch->focus_resource_listener);
+		wl_signal_add(&view->destroy_signal, &touch->focus_view_listener);
 	}
-	seat->touch->focus = view;
+	touch->focus = view;
 }
 
 /**
@@ -1518,7 +1525,7 @@ notify_touch(struct weston_seat *seat, uint32_t time, int touch_id,
 		 * until all touch points are up again. */
 		if (touch->num_tp == 1) {
 			ev = weston_compositor_pick_view(ec, x, y, &sx, &sy);
-			weston_touch_set_focus(seat, ev);
+			weston_touch_set_focus(touch, ev);
 		} else if (touch->focus) {
 			ev = touch->focus;
 			weston_view_from_global_fixed(ev, x, y, &sx, &sy);
@@ -1567,7 +1574,7 @@ notify_touch(struct weston_seat *seat, uint32_t time, int touch_id,
 
 		grab->interface->up(grab, time, touch_id);
 		if (touch->num_tp == 0)
-			weston_touch_set_focus(seat, NULL);
+			weston_touch_set_focus(touch, NULL);
 		break;
 	}
 }
@@ -1581,6 +1588,13 @@ notify_touch_frame(struct weston_seat *seat)
 	grab->interface->frame(grab);
 }
 
+static int
+pointer_cursor_surface_get_label(struct weston_surface *surface,
+				 char *buf, size_t len)
+{
+	return snprintf(buf, len, "cursor");
+}
+
 static void
 pointer_cursor_surface_configure(struct weston_surface *es,
 				 int32_t dx, int32_t dy)
@@ -1634,28 +1648,35 @@ pointer_set_cursor(struct wl_client *client, struct wl_resource *resource,
 	if (pointer->focus_serial - serial > UINT32_MAX / 2)
 		return;
 
-	if (surface && pointer->sprite && surface != pointer->sprite->surface) {
-		if (surface->configure) {
-			wl_resource_post_error(surface->resource,
-					       WL_DISPLAY_ERROR_INVALID_OBJECT,
-					       "surface->configure already "
-					       "set");
-			return;
-		}
+	if (!surface) {
+		if (pointer->sprite)
+			pointer_unmap_sprite(pointer);
+		return;
 	}
 
-	if (pointer->sprite)
-		pointer_unmap_sprite(pointer);
-
-	if (!surface)
+	if (pointer->sprite && pointer->sprite->surface == surface &&
+	    pointer->hotspot_x == x && pointer->hotspot_y == y)
 		return;
 
-	wl_signal_add(&surface->destroy_signal,
-		      &pointer->sprite_destroy_listener);
+	if (!pointer->sprite || pointer->sprite->surface != surface) {
+		if (weston_surface_set_role(surface, "wl_pointer-cursor",
+					    resource,
+					    WL_POINTER_ERROR_ROLE) < 0)
+			return;
+
+		if (pointer->sprite)
+			pointer_unmap_sprite(pointer);
+
+		wl_signal_add(&surface->destroy_signal,
+			      &pointer->sprite_destroy_listener);
+
+		surface->configure = pointer_cursor_surface_configure;
+		surface->configure_private = pointer;
+		weston_surface_set_label_func(surface,
+					    pointer_cursor_surface_get_label);
+		pointer->sprite = weston_view_create(surface);
+	}
 
-	surface->configure = pointer_cursor_surface_configure;
-	surface->configure_private = pointer;
-	pointer->sprite = weston_view_create(surface);
 	pointer->hotspot_x = x;
 	pointer->hotspot_y = y;
 
@@ -1729,7 +1750,7 @@ static const struct wl_keyboard_interface keyboard_interface = {
 	keyboard_release
 };
 
-static int
+static bool
 should_send_modifiers_to_client(struct weston_seat *seat,
 				struct wl_client *client)
 {
@@ -1737,15 +1758,15 @@ should_send_modifiers_to_client(struct weston_seat *seat,
 	    seat->keyboard->focus &&
 	    seat->keyboard->focus->resource &&
 	    wl_resource_get_client(seat->keyboard->focus->resource) == client)
-		return 1;
+		return true;
 
 	if (seat->pointer &&
 	    seat->pointer->focus &&
 	    seat->pointer->focus->surface->resource &&
 	    wl_resource_get_client(seat->pointer->focus->surface->resource) == client)
-		return 1;
+		return true;
 
-	return 0;
+	return false;
 }
 
 static void
@@ -2169,6 +2190,7 @@ WL_EXPORT void
 weston_seat_release_keyboard(struct weston_seat *seat)
 {
 	seat->keyboard_device_count--;
+	assert(seat->keyboard_device_count >= 0);
 	if (seat->keyboard_device_count == 0) {
 		weston_keyboard_set_focus(seat->keyboard, NULL);
 		weston_keyboard_cancel_grab(seat->keyboard);
@@ -2217,6 +2239,11 @@ weston_seat_release_pointer(struct weston_seat *seat)
 
 		weston_pointer_reset_state(pointer);
 		seat_send_updated_caps(seat);
+
+		/* seat->pointer is intentionally not destroyed so that
+		 * a newly attached pointer on this seat will retain
+		 * the previous cursor co-ordinates.
+		 */
 	}
 }
 
@@ -2248,7 +2275,7 @@ weston_seat_release_touch(struct weston_seat *seat)
 {
 	seat->touch_device_count--;
 	if (seat->touch_device_count == 0) {
-		weston_touch_set_focus(seat, NULL);
+		weston_touch_set_focus(seat->touch, NULL);
 		weston_touch_cancel_grab(seat->touch);
 		weston_touch_reset_state(seat->touch);
 		seat_send_updated_caps(seat);
