@@ -114,11 +114,17 @@ weston_launcher_open(struct weston_launcher *launcher,
 	struct weston_launcher_open *message;
 	struct stat s;
 
+	/* We really don't want to be leaking fds to child processes so
+	 * we force this flag here.  If someone comes up with a legitimate
+	 * reason to not CLOEXEC they'll need to unset the flag manually.
+	 */
+	flags |= O_CLOEXEC;
+
 	if (launcher->logind)
 		return weston_logind_open(launcher->logind, path, flags);
 
 	if (launcher->fd == -1) {
-		fd = open(path, flags | O_CLOEXEC);
+		fd = open(path, flags);
 		if (fd == -1)
 			return -1;
 
@@ -190,7 +196,7 @@ void
 weston_launcher_close(struct weston_launcher *launcher, int fd)
 {
 	if (launcher->logind)
-		return weston_logind_close(launcher->logind, fd);
+		weston_logind_close(launcher->logind, fd);
 
 	close(fd);
 }
@@ -342,9 +348,21 @@ setup_tty(struct weston_launcher *launcher, int tty)
 		goto err_close;
 	}
 
+	/*
+	 * SIGRTMIN is used as global VT-acquire+release signal. Note that
+	 * SIGRT* must be tested on runtime, as their exact values are not
+	 * known at compile-time. POSIX requires 32 of them to be available.
+	 */
+	if (SIGRTMIN > SIGRTMAX) {
+		weston_log("not enough RT signals available: %u-%u\n",
+			   SIGRTMIN, SIGRTMAX);
+		ret = -EINVAL;
+		goto err_close;
+	}
+
 	mode.mode = VT_PROCESS;
-	mode.relsig = SIGUSR1;
-	mode.acqsig = SIGUSR1;
+	mode.relsig = SIGRTMIN;
+	mode.acqsig = SIGRTMIN;
 	if (ioctl(launcher->tty, VT_SETMODE, &mode) < 0) {
 		weston_log("failed to take control of vt handling\n");
 		goto err_close;
@@ -352,7 +370,7 @@ setup_tty(struct weston_launcher *launcher, int tty)
 
 	loop = wl_display_get_event_loop(launcher->compositor->wl_display);
 	launcher->vt_source =
-		wl_event_loop_add_signal(loop, SIGUSR1, vt_handler, launcher);
+		wl_event_loop_add_signal(loop, SIGRTMIN, vt_handler, launcher);
 	if (!launcher->vt_source)
 		goto err_close;
 
@@ -374,7 +392,7 @@ weston_launcher_activate_vt(struct weston_launcher *launcher, int vt)
 
 struct weston_launcher *
 weston_launcher_connect(struct weston_compositor *compositor, int tty,
-			const char *seat_id)
+			const char *seat_id, bool sync_drm)
 {
 	struct weston_launcher *launcher;
 	struct wl_event_loop *loop;
@@ -406,7 +424,7 @@ weston_launcher_connect(struct weston_compositor *compositor, int tty,
 		}
 	} else {
 		r = weston_logind_connect(&launcher->logind, compositor,
-					  seat_id, tty);
+					  seat_id, tty, sync_drm);
 		if (r < 0) {
 			launcher->logind = NULL;
 			if (geteuid() == 0) {
