@@ -337,8 +337,7 @@ apply_opaque_region(struct wl_shm_buffer *buffer,
 	for (y = 0; y < height; y++) {
 		for (x = 0; x < width; x++) {
 			int i = y * stride / 4 + x;
-			pixman_box32_t box;
-			if (pixman_region32_contains_point (opaque_region, x, y, &box)) {
+			if (pixman_region32_contains_point (opaque_region, x, y, NULL)) {
 				dst[i] = src[i] | 0xff000000;
 			} else {
 				dst[i] = src[i];
@@ -492,12 +491,11 @@ rpir_surface_create(struct rpi_renderer *renderer)
 {
 	struct rpir_surface *surface;
 
-	surface = calloc(1, sizeof *surface);
-	if (!surface)
+	surface = zalloc(sizeof *surface);
+	if (surface == NULL)
 		return NULL;
 
 	wl_list_init(&surface->views);
-	surface->visible_views = 0;
 	surface->single_buffer = renderer->single_buffer;
 	surface->enable_opaque_regions = renderer->enable_opaque_regions;
 	rpi_resource_init(&surface->resources[0]);
@@ -577,8 +575,8 @@ rpir_view_create(struct rpir_surface *surface)
 {
 	struct rpir_view *view;
 
-	view = calloc(1, sizeof *view);
-	if (!view)
+	view = zalloc(sizeof *view);
+	if (view == NULL)
 		return NULL;
 
 	view->surface = surface;
@@ -1327,10 +1325,11 @@ output_compute_matrix(struct weston_output *base)
 {
 	struct rpir_output *output = to_rpir_output(base);
 	struct weston_matrix *matrix = &output->matrix;
+#ifdef SURFACE_TRANSFORM
 	const float half_w = 0.5f * base->width;
 	const float half_h = 0.5f * base->height;
+#endif
 	float mag;
-	float dx, dy;
 
 	weston_matrix_init(matrix);
 	weston_matrix_translate(matrix, -base->x, -base->y, 0.0f);
@@ -1372,13 +1371,10 @@ output_compute_matrix(struct weston_output *base)
 #endif
 
 	if (base->zoom.active) {
-		/* The base->zoom stuff is in GL coordinate system */
 		mag = 1.0f / (1.0f - base->zoom.spring_z.current);
-		dx = -(base->zoom.trans_x + 1.0f) * half_w;
-		dy = -(base->zoom.trans_y + 1.0f) * half_h;
-		weston_matrix_translate(matrix, dx, dy, 0.0f);
+		weston_matrix_translate(matrix, base->zoom.trans_x,
+					base->zoom.trans_y, 0.0f);
 		weston_matrix_scale(matrix, mag, mag, 1.0f);
-		weston_matrix_translate(matrix, half_w, half_h, 0.0f);
 	}
 }
 
@@ -1550,7 +1546,7 @@ rpi_renderer_attach(struct weston_surface *base, struct weston_buffer *buffer)
 		surface->buffer_type = BUFFER_TYPE_EGL;
 
 		if(surface->egl_back == NULL)
-			surface->egl_back = calloc(1, sizeof *surface->egl_back);
+			surface->egl_back = zalloc(sizeof *surface->egl_back);
 
 		weston_buffer_reference(&surface->egl_back->buffer_ref, buffer);
 		surface->egl_back->resource_handle =
@@ -1726,7 +1722,7 @@ rpi_renderer_create(struct weston_compositor *compositor,
 
 	weston_log("Initializing the DispmanX compositing renderer\n");
 
-	renderer = calloc(1, sizeof *renderer);
+	renderer = zalloc(sizeof *renderer);
 	if (renderer == NULL)
 		return -1;
 
@@ -1798,8 +1794,8 @@ rpi_renderer_output_create(struct weston_output *base,
 
 	assert(base->renderer_state == NULL);
 
-	output = calloc(1, sizeof *output);
-	if (!output)
+	output = zalloc(sizeof *output);
+	if (output == NULL)
 		return -1;
 
 	output->display = display;
