@@ -63,39 +63,6 @@ weston_zoom_pick_seat(struct weston_compositor *compositor)
 			    struct weston_seat, link);
 }
 
-
-static void
-weston_zoom_frame_xy(struct weston_animation *animation,
-		struct weston_output *output, uint32_t msecs)
-{
-	struct weston_seat *seat = weston_zoom_pick_seat(output->compositor);
-	wl_fixed_t x, y;
-
-	if (animation->frame_counter <= 1)
-		output->zoom.spring_xy.timestamp = msecs;
-
-	weston_spring_update(&output->zoom.spring_xy, msecs);
-
-	x = output->zoom.from.x - ((output->zoom.from.x - output->zoom.to.x) *
-						output->zoom.spring_xy.current);
-	y = output->zoom.from.y - ((output->zoom.from.y - output->zoom.to.y) *
-						output->zoom.spring_xy.current);
-
-	output->zoom.current.x = x;
-	output->zoom.current.y = y;
-
-	if (weston_spring_done(&output->zoom.spring_xy)) {
-		output->zoom.spring_xy.current = output->zoom.spring_xy.target;
-		output->zoom.current.x = seat->pointer->x;
-		output->zoom.current.y = seat->pointer->y;
-		wl_list_remove(&animation->link);
-		wl_list_init(&animation->link);
-	}
-
-	output->dirty = 1;
-	weston_output_damage(output);
-}
-
 static void
 zoom_area_center_from_pointer(struct weston_output *output,
 				wl_fixed_t *x, wl_fixed_t *y)
@@ -106,55 +73,44 @@ zoom_area_center_from_pointer(struct weston_output *output,
 	wl_fixed_t w = wl_fixed_from_int(output->width);
 	wl_fixed_t h = wl_fixed_from_int(output->height);
 
-	*x -= ((((*x - offset_x) / (float) w) - 0.5) * (w * (1.0 - level)));
-	*y -= ((((*y - offset_y) / (float) h) - 0.5) * (h * (1.0 - level)));
+	*x = (*x - offset_x) * level + w / 2;
+	*y = (*y - offset_y) * level + h / 2;
 }
 
 static void
 weston_output_update_zoom_transform(struct weston_output *output)
 {
 	float global_x, global_y;
-	wl_fixed_t x = output->zoom.current.x;
+	wl_fixed_t x = output->zoom.current.x; /* global pointer coords */
 	wl_fixed_t y = output->zoom.current.y;
-	float trans_min, trans_max;
-	float ratio, level;
+	float level;
 
 	level = output->zoom.spring_z.current;
-	ratio = 1 / level;
 
 	if (!output->zoom.active || level > output->zoom.max_level ||
 	    level == 0.0f)
 		return;
 
-	if (wl_list_empty(&output->zoom.animation_xy.link))
-		zoom_area_center_from_pointer(output, &x, &y);
+	zoom_area_center_from_pointer(output, &x, &y);
 
 	global_x = wl_fixed_to_double(x);
 	global_y = wl_fixed_to_double(y);
 
-	output->zoom.trans_x =
-		((((global_x - output->x) / output->width) *
-		(level * 2)) - level) * ratio;
-	output->zoom.trans_y =
-		((((global_y - output->y) / output->height) *
-		(level * 2)) - level) * ratio;
+	output->zoom.trans_x = global_x - output->width / 2;
+	output->zoom.trans_y = global_y - output->height / 2;
 
-	trans_max = level * 2 - level;
-	trans_min = -trans_max;
-
-	/* Clip zoom area to output */
-	if (output->zoom.trans_x > trans_max)
-		output->zoom.trans_x = trans_max;
-	else if (output->zoom.trans_x < trans_min)
-		output->zoom.trans_x = trans_min;
-	if (output->zoom.trans_y > trans_max)
-		output->zoom.trans_y = trans_max;
-	else if (output->zoom.trans_y < trans_min)
-		output->zoom.trans_y = trans_min;
+	if (output->zoom.trans_x < 0)
+		output->zoom.trans_x = 0;
+	if (output->zoom.trans_y < 0)
+		output->zoom.trans_y = 0;
+	if (output->zoom.trans_x > level * output->width)
+		output->zoom.trans_x = level * output->width;
+	if (output->zoom.trans_y > level * output->height)
+		output->zoom.trans_y = level * output->height;
 }
 
 static void
-weston_zoom_transition(struct weston_output *output, wl_fixed_t x, wl_fixed_t y)
+weston_zoom_transition(struct weston_output *output)
 {
 	if (output->zoom.level != output->zoom.spring_z.current) {
 		output->zoom.spring_z.target = output->zoom.level;
@@ -173,20 +129,11 @@ WL_EXPORT void
 weston_output_update_zoom(struct weston_output *output)
 {
 	struct weston_seat *seat = weston_zoom_pick_seat(output->compositor);
-	wl_fixed_t x = seat->pointer->x;
-	wl_fixed_t y = seat->pointer->y;
 
-	zoom_area_center_from_pointer(output, &x, &y);
+	output->zoom.current.x = seat->pointer->x;
+	output->zoom.current.y = seat->pointer->y;
 
-	if (wl_list_empty(&output->zoom.animation_xy.link)) {
-		output->zoom.current.x = seat->pointer->x;
-		output->zoom.current.y = seat->pointer->y;
-	} else {
-		output->zoom.to.x = x;
-		output->zoom.to.y = y;
-	}
-
-	weston_zoom_transition(output, x, y);
+	weston_zoom_transition(output);
 	weston_output_update_zoom_transform(output);
 }
 
@@ -228,9 +175,5 @@ weston_output_init_zoom(struct weston_output *output)
 	output->zoom.spring_z.friction = 1000;
 	output->zoom.animation_z.frame = weston_zoom_frame_z;
 	wl_list_init(&output->zoom.animation_z.link);
-	weston_spring_init(&output->zoom.spring_xy, 250.0, 0.0, 0.0);
-	output->zoom.spring_xy.friction = 1000;
-	output->zoom.animation_xy.frame = weston_zoom_frame_xy;
-	wl_list_init(&output->zoom.animation_xy.link);
 	output->zoom.motion_listener.notify = motion;
 }
