@@ -24,7 +24,7 @@
 
 #include <string.h>
 #include <stdio.h>
-#include <linux/input.h>
+
 #include "weston-test-client-helper.h"
 #include "text-client-protocol.h"
 
@@ -169,7 +169,7 @@ TEST(text_test)
 	struct wl_text_input *text_input;
 	struct text_input_state state;
 
-	client = client_create(100, 100, 100, 100);
+	client = create_client_and_test_surface(100, 100, 100, 100);
 	assert(client);
 
 	factory = NULL;
@@ -187,7 +187,7 @@ TEST(text_test)
 	wl_text_input_add_listener(text_input, &text_input_listener, &state);
 
 	/* Make sure our test surface has keyboard focus. */
-	wl_test_activate_surface(client->test->wl_test,
+	weston_test_activate_surface(client->test->weston_test,
 				 client->surface->wl_surface);
 	client_roundtrip(client);
 	assert(client->input->keyboard->focus == client->surface);
@@ -210,7 +210,7 @@ TEST(text_test)
 	assert(state.activated == 2 && state.deactivated == 1);
 
 	/* Take keyboard focus away and verify we get leave event. */
-	wl_test_activate_surface(client->test->wl_test, NULL);
+	weston_test_activate_surface(client->test->weston_test, NULL);
 	client_roundtrip(client);
 	assert(state.activated == 2 && state.deactivated == 2);
 }
