@@ -94,6 +94,10 @@ struct motif_wm_hints {
 #define MWM_DECOR_MINIMIZE      (1L << 5)
 #define MWM_DECOR_MAXIMIZE      (1L << 6)
 
+#define MWM_DECOR_EVERYTHING \
+	(MWM_DECOR_BORDER | MWM_DECOR_RESIZEH | MWM_DECOR_TITLE | \
+	 MWM_DECOR_MENU | MWM_DECOR_MINIMIZE | MWM_DECOR_MAXIMIZE)
+
 #define MWM_INPUT_MODELESS 0
 #define MWM_INPUT_PRIMARY_APPLICATION_MODAL 1
 #define MWM_INPUT_SYSTEM_MODAL 2
@@ -144,6 +148,8 @@ struct weston_wm_window {
 	int fullscreen;
 	int has_alpha;
 	int delete_window;
+	int maximized_vert;
+	int maximized_horz;
 	struct wm_size_hints size_hints;
 	struct motif_wm_hints motif_hints;
 	struct wl_list link;
@@ -193,6 +199,15 @@ wm_log_continue(const char *fmt, ...)
 #endif
 }
 
+static bool __attribute__ ((warn_unused_result))
+wm_lookup_window(struct weston_wm *wm, xcb_window_t hash,
+		 struct weston_wm_window **window)
+{
+	*window = hash_table_lookup(wm->window_hash, hash);
+	if (*window)
+		return true;
+	return false;
+}
 
 const char *
 get_atom_name(xcb_connection_t *c, xcb_atom_t atom)
@@ -401,6 +416,7 @@ weston_wm_window_read_properties(struct weston_wm_window *window)
 	uint32_t *xid;
 	xcb_atom_t *atom;
 	uint32_t i;
+	char name[1024];
 
 	if (!window->properties_dirty)
 		return;
@@ -413,7 +429,7 @@ weston_wm_window_read_properties(struct weston_wm_window *window)
 					     props[i].atom,
 					     XCB_ATOM_ANY, 0, 2048);
 
-	window->decorate = !window->override_redirect;
+	window->decorate = window->override_redirect ? 0 : MWM_DECOR_EVERYTHING;
 	window->size_hints.flags = 0;
 	window->motif_hints.flags = 0;
 	window->delete_window = 0;
@@ -445,8 +461,9 @@ weston_wm_window_read_properties(struct weston_wm_window *window)
 			break;
 		case XCB_ATOM_WINDOW:
 			xid = xcb_get_property_value(reply);
-			*(struct weston_wm_window **) p =
-				hash_table_lookup(wm->window_hash, *xid);
+			if (!wm_lookup_window(wm, *xid, p))
+				weston_log("XCB_ATOM_WINDOW contains window"
+					   " id not found in hash table.\n");
 			break;
 		case XCB_ATOM_CARDINAL:
 		case XCB_ATOM_ATOM:
@@ -456,10 +473,10 @@ weston_wm_window_read_properties(struct weston_wm_window *window)
 		case TYPE_WM_PROTOCOLS:
 			atom = xcb_get_property_value(reply);
 			for (i = 0; i < reply->value_len; i++)
-				if (atom[i] == wm->atom.wm_delete_window)
+				if (atom[i] == wm->atom.wm_delete_window) {
 					window->delete_window = 1;
-			break;
-
+					break;
+				}
 			break;
 		case TYPE_WM_NORMAL_HINTS:
 			memcpy(&window->size_hints,
@@ -469,17 +486,28 @@ weston_wm_window_read_properties(struct weston_wm_window *window)
 		case TYPE_NET_WM_STATE:
 			window->fullscreen = 0;
 			atom = xcb_get_property_value(reply);
-			for (i = 0; i < reply->value_len; i++)
+			for (i = 0; i < reply->value_len; i++) {
 				if (atom[i] == wm->atom.net_wm_state_fullscreen)
 					window->fullscreen = 1;
+				if (atom[i] == wm->atom.net_wm_state_maximized_vert)
+					window->maximized_vert = 1;
+				if (atom[i] == wm->atom.net_wm_state_maximized_horz)
+					window->maximized_horz = 1;
+			}
 			break;
 		case TYPE_MOTIF_WM_HINTS:
 			memcpy(&window->motif_hints,
 			       xcb_get_property_value(reply),
 			       sizeof window->motif_hints);
-			if (window->motif_hints.flags & MWM_HINTS_DECORATIONS)
-				window->decorate =
-					window->motif_hints.decorations > 0;
+			if (window->motif_hints.flags & MWM_HINTS_DECORATIONS) {
+				if (window->motif_hints.decorations & MWM_DECOR_ALL)
+					/* MWM_DECOR_ALL means all except the other values listed. */
+					window->decorate =
+						MWM_DECOR_EVERYTHING & (~window->motif_hints.decorations);
+				else
+					window->decorate =
+						window->motif_hints.decorations;
+			}
 			break;
 		default:
 			break;
@@ -487,10 +515,28 @@ weston_wm_window_read_properties(struct weston_wm_window *window)
 		free(reply);
 	}
 
+	if (window->pid > 0) {
+		gethostname(name, sizeof(name));
+		for (i = 0; i < sizeof(name); i++) {
+			if (name[i] == '\0')
+				break;
+		}
+		if (i == sizeof(name))
+			name[0] = '\0'; /* ignore stupid hostnames */
+
+		/* this is only one heuristic to guess the PID of a client is
+		* valid, assuming it's compliant with icccm and ewmh.
+		* Non-compliants and remote applications of course fail. */
+		if (!window->machine || strcmp(window->machine, name))
+			window->pid = 0;
+	}
+
 	if (window->shsurf && window->name)
 		shell_interface->set_title(window->shsurf, window->name);
 	if (window->frame && window->name)
 		frame_set_title(window->frame, window->name);
+	if (window->shsurf && window->pid > 0)
+		shell_interface->set_pid(window->shsurf, window->pid);
 }
 
 static void
@@ -568,7 +614,8 @@ weston_wm_handle_configure_request(struct weston_wm *wm, xcb_generic_event_t *ev
 	       configure_request->x, configure_request->y,
 	       configure_request->width, configure_request->height);
 
-	window = hash_table_lookup(wm->window_hash, configure_request->window);
+	if (!wm_lookup_window(wm, configure_request->window, &window))
+		return;
 
 	if (window->fullscreen) {
 		weston_wm_window_send_configure_notify(window);
@@ -634,7 +681,9 @@ weston_wm_handle_configure_notify(struct weston_wm *wm, xcb_generic_event_t *eve
 	       configure_notify->x, configure_notify->y,
 	       configure_notify->width, configure_notify->height);
 
-	window = hash_table_lookup(wm->window_hash, configure_notify->window);
+	if (!wm_lookup_window(wm, configure_notify->window, &window))
+		return;
+
 	window->x = configure_notify->x;
 	window->y = configure_notify->y;
 	if (window->override_redirect) {
@@ -651,17 +700,10 @@ weston_wm_kill_client(struct wl_listener *listener, void *data)
 {
 	struct weston_surface *surface = data;
 	struct weston_wm_window *window = get_wm_window(surface);
-	char name[1024];
-
 	if (!window)
 		return;
 
-	gethostname(name, 1024);
-
-	/* this is only one heuristic to guess the PID of a client is valid,
-	 * assuming it's compliant with icccm and ewmh. Non-compliants and
-	 * remote applications of course fail. */
-	if (!strcmp(window->machine, name) && window->pid != 0)
+	if (window->pid > 0)
 		kill(window->pid, SIGKILL);
 }
 
@@ -701,6 +743,8 @@ weston_wm_window_activate(struct wl_listener *listener, void *data)
 	}
 
 	if (window) {
+		uint32_t values[1];
+
 		if (window->override_redirect)
 			return;
 
@@ -717,6 +761,10 @@ weston_wm_window_activate(struct wl_listener *listener, void *data)
 
 		xcb_set_input_focus (wm->conn, XCB_INPUT_FOCUS_POINTER_ROOT,
 				     window->id, XCB_TIME_CURRENT_TIME);
+
+		values[0] = XCB_STACK_MODE_ABOVE;
+		xcb_configure_window (wm->conn, window->frame_id,
+				      XCB_CONFIG_WINDOW_STACK_MODE, values);
 	} else {
 		xcb_set_input_focus (wm->conn,
 				     XCB_INPUT_FOCUS_POINTER_ROOT,
@@ -789,12 +837,16 @@ static void
 weston_wm_window_set_net_wm_state(struct weston_wm_window *window)
 {
 	struct weston_wm *wm = window->wm;
-	uint32_t property[1];
+	uint32_t property[3];
 	int i;
 
 	i = 0;
 	if (window->fullscreen)
 		property[i++] = wm->atom.net_wm_state_fullscreen;
+	if (window->maximized_vert)
+		property[i++] = wm->atom.net_wm_state_maximized_vert;
+	if (window->maximized_horz)
+		property[i++] = wm->atom.net_wm_state_maximized_horz;
 
 	xcb_change_property(wm->conn,
 			    XCB_PROP_MODE_REPLACE,
@@ -811,10 +863,14 @@ weston_wm_window_create_frame(struct weston_wm_window *window)
 	struct weston_wm *wm = window->wm;
 	uint32_t values[3];
 	int x, y, width, height;
+	int buttons = FRAME_BUTTON_CLOSE;
+
+	if (window->decorate & MWM_DECOR_MAXIMIZE)
+		buttons |= FRAME_BUTTON_MAXIMIZE;
 
 	window->frame = frame_create(window->wm->theme,
 				     window->width, window->height,
-				     FRAME_BUTTON_CLOSE, window->name);
+				     buttons, window->name);
 	frame_resize_inside(window->frame, window->width, window->height);
 
 	weston_wm_window_get_frame_size(window, &width, &height);
@@ -863,6 +919,29 @@ weston_wm_window_create_frame(struct weston_wm_window *window)
 	hash_table_insert(wm->window_hash, window->frame_id, window);
 }
 
+/*
+ * Sets the _NET_WM_DESKTOP property for the window to 'desktop'.
+ * Passing a <0 desktop value deletes the property.
+ */
+static void
+weston_wm_window_set_virtual_desktop(struct weston_wm_window *window,
+                                     int desktop)
+{
+	if (desktop >= 0) {
+		xcb_change_property(window->wm->conn,
+		                    XCB_PROP_MODE_REPLACE,
+		                    window->id,
+		                    window->wm->atom.net_wm_desktop,
+		                    XCB_ATOM_CARDINAL,
+		                    32, /* format */
+		                    1, &desktop);
+	} else {
+		xcb_delete_property(window->wm->conn,
+		                    window->id,
+		                    window->wm->atom.net_wm_desktop);
+	}
+}
+
 static void
 weston_wm_handle_map_request(struct weston_wm *wm, xcb_generic_event_t *event)
 {
@@ -876,7 +955,8 @@ weston_wm_handle_map_request(struct weston_wm *wm, xcb_generic_event_t *event)
 		return;
 	}
 
-	window = hash_table_lookup(wm->window_hash, map_request->window);
+	if (!wm_lookup_window(wm, map_request->window, &window))
+		return;
 
 	weston_wm_window_read_properties(window);
 
@@ -888,6 +968,7 @@ weston_wm_handle_map_request(struct weston_wm *wm, xcb_generic_event_t *event)
 
 	weston_wm_window_set_wm_state(window, ICCCM_NORMAL_STATE);
 	weston_wm_window_set_net_wm_state(window);
+	weston_wm_window_set_virtual_desktop(window, 0);
 
 	xcb_map_window(wm->conn, map_request->window);
 	xcb_map_window(wm->conn, window->frame_id);
@@ -927,7 +1008,17 @@ weston_wm_handle_unmap_notify(struct weston_wm *wm, xcb_generic_event_t *event)
 		 * as it may come in after we've destroyed the window. */
 		return;
 
-	window = hash_table_lookup(wm->window_hash, unmap_notify->window);
+	if (!wm_lookup_window(wm, unmap_notify->window, &window))
+		return;
+
+	if (window->surface_id) {
+		/* Make sure we're not on the unpaired surface list or we
+		 * could be assigned a surface during surface creation that
+		 * was mapped before this unmap request.
+		 */
+		wl_list_remove(&window->link);
+		window->surface_id = 0;
+	}
 	if (wm->focus_window == window)
 		wm->focus_window = NULL;
 	if (window->surface)
@@ -935,6 +1026,10 @@ weston_wm_handle_unmap_notify(struct weston_wm *wm, xcb_generic_event_t *event)
 	window->surface = NULL;
 	window->shsurf = NULL;
 	window->view = NULL;
+
+	weston_wm_window_set_wm_state(window, ICCCM_WITHDRAWN_STATE);
+	weston_wm_window_set_virtual_desktop(window, -1);
+
 	xcb_unmap_window(wm->conn, window->frame_id);
 }
 
@@ -973,9 +1068,7 @@ weston_wm_window_draw_decoration(void *data)
 		cairo_set_source_rgba(cr, 0, 0, 0, 0);
 		cairo_paint(cr);
 
-		cairo_set_operator(cr, CAIRO_OPERATOR_OVER);
-		cairo_set_source_rgba(cr, 0, 0, 0, 0.45);
-		tile_mask(cr, t->shadow, 2, 2, width + 8, height + 8, 64, 64);
+		render_shadow(cr, t->shadow, 2, 2, width + 8, height + 8, 64, 64);
 	}
 
 	cairo_destroy(cr);
@@ -998,14 +1091,14 @@ weston_wm_window_draw_decoration(void *data)
 
 		pixman_region32_fini(&window->surface->pending.input);
 
-		if (window->fullscreen) {
-			input_x = 0;
-			input_y = 0;
-			input_w = window->width;
-			input_h = window->height;
-		} else if (window->decorate) {
+		if (window->decorate && !window->fullscreen) {
 			frame_input_rect(window->frame, &input_x, &input_y,
 					 &input_w, &input_h);
+		} else {
+			input_x = x;
+			input_y = y;
+			input_w = width;
+			input_h = height;
 		}
 
 		pixman_region32_init_rect(&window->surface->pending.input,
@@ -1054,8 +1147,7 @@ weston_wm_handle_property_notify(struct weston_wm *wm, xcb_generic_event_t *even
 		(xcb_property_notify_event_t *) event;
 	struct weston_wm_window *window;
 
-	window = hash_table_lookup(wm->window_hash, property_notify->window);
-	if (!window)
+	if (!wm_lookup_window(wm, property_notify->window, &window))
 		return;
 
 	window->properties_dirty = 1;
@@ -1125,6 +1217,7 @@ weston_wm_window_destroy(struct weston_wm_window *window)
 		xcb_reparent_window(wm->conn, window->id, wm->wm_window, 0, 0);
 		xcb_destroy_window(wm->conn, window->frame_id);
 		weston_wm_window_set_wm_state(window, ICCCM_WITHDRAWN_STATE);
+		weston_wm_window_set_virtual_desktop(window, -1);
 		hash_table_remove(wm->window_hash, window->frame_id);
 		window->frame_id = XCB_WINDOW_NONE;
 	}
@@ -1132,6 +1225,9 @@ weston_wm_window_destroy(struct weston_wm_window *window)
 	if (window->surface_id)
 		wl_list_remove(&window->link);
 
+	if (window->surface)
+		wl_list_remove(&window->surface_destroy_listener.link);
+
 	hash_table_remove(window->wm->window_hash, window->id);
 	free(window);
 }
@@ -1172,7 +1268,9 @@ weston_wm_handle_destroy_notify(struct weston_wm *wm, xcb_generic_event_t *event
 	if (our_resource(wm, destroy_notify->window))
 		return;
 
-	window = hash_table_lookup(wm->window_hash, destroy_notify->window);
+	if (!wm_lookup_window(wm, destroy_notify->window, &window))
+		return;
+
 	weston_wm_window_destroy(window);
 }
 
@@ -1193,8 +1291,8 @@ weston_wm_handle_reparent_notify(struct weston_wm *wm, xcb_generic_event_t *even
 					reparent_notify->x, reparent_notify->y,
 					reparent_notify->override_redirect);
 	} else if (!our_resource(wm, reparent_notify->parent)) {
-		window = hash_table_lookup(wm->window_hash,
-					   reparent_notify->window);
+		if (!wm_lookup_window(wm, reparent_notify->window, &window))
+			return;
 		weston_wm_window_destroy(window);
 	}
 }
@@ -1214,8 +1312,8 @@ weston_wm_pick_seat_for_window(struct weston_wm_window *window)
 
 	seat = NULL;
 	wl_list_for_each(s, &wm->server->compositor->seat_list, link) {
-		if (s->pointer != NULL &&
-		    s->pointer->focus == window->view &&
+		if (s->pointer != NULL && s->pointer->focus &&
+		    s->pointer->focus->surface == window->surface &&
 		    s->pointer->button_count > 0 &&
 		    (seat == NULL ||
 		     s->pointer->grab_serial -
@@ -1248,7 +1346,8 @@ weston_wm_window_handle_moveresize(struct weston_wm_window *window,
 		&wm->server->compositor->shell_interface;
 
 	if (seat == NULL || seat->pointer->button_count != 1
-	    || !window->view || seat->pointer->focus != window->view)
+	    || !seat->pointer->focus
+	    || seat->pointer->focus->surface != window->surface)
 		return;
 
 	detail = client_message->data.data32[2];
@@ -1303,6 +1402,28 @@ update_state(int action, int *state)
 static void
 weston_wm_window_configure(void *data);
 
+static void
+weston_wm_window_set_toplevel(struct weston_wm_window *window)
+{
+	struct weston_shell_interface *shell_interface =
+		&window->wm->server->compositor->shell_interface;
+
+	shell_interface->set_toplevel(window->shsurf);
+	window->width = window->saved_width;
+	window->height = window->saved_height;
+	if (window->frame)
+		frame_resize_inside(window->frame,
+					window->width,
+					window->height);
+	weston_wm_window_configure(window);
+}
+
+static inline bool
+weston_wm_window_is_maximized(struct weston_wm_window *window)
+{
+	return window->maximized_horz && window->maximized_vert;
+}
+
 static void
 weston_wm_window_handle_state(struct weston_wm_window *window,
 			      xcb_client_message_event_t *client_message)
@@ -1311,6 +1432,7 @@ weston_wm_window_handle_state(struct weston_wm_window *window,
 	struct weston_shell_interface *shell_interface =
 		&wm->server->compositor->shell_interface;
 	uint32_t action, property;
+	int maximized = weston_wm_window_is_maximized(window);
 
 	action = client_message->data.data32[0];
 	property = client_message->data.data32[1];
@@ -1328,15 +1450,26 @@ weston_wm_window_handle_state(struct weston_wm_window *window,
 								0, NULL);
 		} else {
 			if (window->shsurf)
-				shell_interface->set_toplevel(window->shsurf);
+				weston_wm_window_set_toplevel(window);
+		}
+	} else {
+		if (property == wm->atom.net_wm_state_maximized_vert &&
+		    update_state(action, &window->maximized_vert))
+			weston_wm_window_set_net_wm_state(window);
+		if (property == wm->atom.net_wm_state_maximized_horz &&
+		    update_state(action, &window->maximized_horz))
+			weston_wm_window_set_net_wm_state(window);
 
-			window->width = window->saved_width;
-			window->height = window->saved_height;
-			if (window->frame)
-				frame_resize_inside(window->frame,
-						    window->width,
-						    window->height);
-			weston_wm_window_configure(window);
+		if (maximized != weston_wm_window_is_maximized(window)) {
+			if (weston_wm_window_is_maximized(window)) {
+				window->saved_width = window->width;
+				window->saved_height = window->height;
+
+				if (window->shsurf)
+					shell_interface->set_maximized(window->shsurf);
+			} else if (window->shsurf) {
+				weston_wm_window_set_toplevel(window);
+			}
 		}
 	}
 }
@@ -1397,8 +1530,6 @@ weston_wm_handle_client_message(struct weston_wm *wm,
 		(xcb_client_message_event_t *) event;
 	struct weston_wm_window *window;
 
-	window = hash_table_lookup(wm->window_hash, client_message->window);
-
 	wm_log("XCB_CLIENT_MESSAGE (%s %d %d %d %d %d win %d)\n",
 	       get_atom_name(wm->conn, client_message->type),
 	       client_message->data.data32[0],
@@ -1411,7 +1542,7 @@ weston_wm_handle_client_message(struct weston_wm *wm,
 	/* The window may get created and destroyed before we actually
 	 * handle the message.  If it doesn't exist, bail.
 	 */
-	if (!window)
+	if (!wm_lookup_window(wm, client_message->window, &window))
 		return;
 
 	if (client_message->type == wm->atom.net_wm_moveresize)
@@ -1627,8 +1758,8 @@ weston_wm_handle_button(struct weston_wm *wm, xcb_generic_event_t *event)
 	       button->response_type == XCB_BUTTON_PRESS ?
 	       "PRESS" : "RELEASE", button->detail);
 
-	window = hash_table_lookup(wm->window_hash, button->event);
-	if (!window || !window->decorate)
+	if (!wm_lookup_window(wm, button->event, &window) ||
+	    !window->decorate)
 		return;
 
 	if (button->detail != 1 && button->detail != 2)
@@ -1668,6 +1799,19 @@ weston_wm_handle_button(struct weston_wm *wm, xcb_generic_event_t *event)
 		weston_wm_window_close(window, button->time);
 		frame_status_clear(window->frame, FRAME_STATUS_CLOSE);
 	}
+
+	if (frame_status(window->frame) & FRAME_STATUS_MAXIMIZE) {
+		window->maximized_horz = !window->maximized_horz;
+		window->maximized_vert = !window->maximized_vert;
+		if (weston_wm_window_is_maximized(window)) {
+			window->saved_width = window->width;
+			window->saved_height = window->height;
+			shell_interface->set_maximized(window->shsurf);
+		} else {
+			weston_wm_window_set_toplevel(window);
+		}
+		frame_status_clear(window->frame, FRAME_STATUS_MAXIMIZE);
+	}
 }
 
 static void
@@ -1678,8 +1822,8 @@ weston_wm_handle_motion(struct weston_wm *wm, xcb_generic_event_t *event)
 	enum theme_location location;
 	int cursor;
 
-	window = hash_table_lookup(wm->window_hash, motion->event);
-	if (!window || !window->decorate)
+	if (!wm_lookup_window(wm, motion->event, &window) ||
+	    !window->decorate)
 		return;
 
 	location = frame_pointer_motion(window->frame, NULL,
@@ -1699,8 +1843,8 @@ weston_wm_handle_enter(struct weston_wm *wm, xcb_generic_event_t *event)
 	enum theme_location location;
 	int cursor;
 
-	window = hash_table_lookup(wm->window_hash, enter->event);
-	if (!window || !window->decorate)
+	if (!wm_lookup_window(wm, enter->event, &window) ||
+	    !window->decorate)
 		return;
 
 	location = frame_pointer_enter(window->frame, NULL,
@@ -1718,8 +1862,8 @@ weston_wm_handle_leave(struct weston_wm *wm, xcb_generic_event_t *event)
 	xcb_leave_notify_event_t *leave = (xcb_leave_notify_event_t *) event;
 	struct weston_wm_window *window;
 
-	window = hash_table_lookup(wm->window_hash, leave->event);
-	if (!window || !window->decorate)
+	if (!wm_lookup_window(wm, leave->event, &window) ||
+	    !window->decorate)
 		return;
 
 	frame_pointer_leave(window->frame, NULL);
@@ -1856,9 +2000,12 @@ weston_wm_get_resources(struct weston_wm *wm)
 		{ "_NET_WM_PID",	F(atom.net_wm_pid) },
 		{ "_NET_WM_ICON",	F(atom.net_wm_icon) },
 		{ "_NET_WM_STATE",	F(atom.net_wm_state) },
+		{ "_NET_WM_STATE_MAXIMIZED_VERT", F(atom.net_wm_state_maximized_vert) },
+		{ "_NET_WM_STATE_MAXIMIZED_HORZ", F(atom.net_wm_state_maximized_horz) },
 		{ "_NET_WM_STATE_FULLSCREEN", F(atom.net_wm_state_fullscreen) },
 		{ "_NET_WM_USER_TIME", F(atom.net_wm_user_time) },
 		{ "_NET_WM_ICON_NAME", F(atom.net_wm_icon_name) },
+		{ "_NET_WM_DESKTOP", F(atom.net_wm_desktop) },
 		{ "_NET_WM_WINDOW_TYPE", F(atom.net_wm_window_type) },
 
 		{ "_NET_WM_WINDOW_TYPE_DESKTOP", F(atom.net_wm_window_type_desktop) },
@@ -2032,7 +2179,7 @@ weston_wm_create(struct weston_xserver *wxs, int fd)
 	struct wl_event_loop *loop;
 	xcb_screen_iterator_t s;
 	uint32_t values[1];
-	xcb_atom_t supported[3];
+	xcb_atom_t supported[5];
 
 	wm = zalloc(sizeof *wm);
 	if (wm == NULL)
@@ -2083,6 +2230,8 @@ weston_wm_create(struct weston_xserver *wxs, int fd)
 	supported[0] = wm->atom.net_wm_moveresize;
 	supported[1] = wm->atom.net_wm_state;
 	supported[2] = wm->atom.net_wm_state_fullscreen;
+	supported[3] = wm->atom.net_wm_state_maximized_vert;
+	supported[4] = wm->atom.net_wm_state_maximized_horz;
 	xcb_change_property(wm->conn,
 			    XCB_PROP_MODE_REPLACE,
 			    wm->screen->root,
@@ -2135,6 +2284,7 @@ weston_wm_destroy(struct weston_wm *wm)
 	wl_list_remove(&wm->activate_listener.link);
 	wl_list_remove(&wm->kill_listener.link);
 	wl_list_remove(&wm->transform_listener.link);
+	wl_list_remove(&wm->create_surface_listener.link);
 
 	free(wm);
 }
@@ -2274,6 +2424,19 @@ legacy_fullscreen(struct weston_wm *wm,
 	return 0;
 }
 
+static bool
+weston_wm_window_type_inactive(struct weston_wm_window *window)
+{
+	struct weston_wm *wm = window->wm;
+
+	return window->type == wm->atom.net_wm_window_type_tooltip ||
+	       window->type == wm->atom.net_wm_window_type_dropdown ||
+	       window->type == wm->atom.net_wm_window_type_dnd ||
+	       window->type == wm->atom.net_wm_window_type_combo ||
+	       window->type == wm->atom.net_wm_window_type_popup ||
+	       window->type == wm->atom.net_wm_window_type_utility;
+}
+
 static void
 xserver_map_shell_surface(struct weston_wm_window *window,
 			  struct weston_surface *surface)
@@ -2283,6 +2446,7 @@ xserver_map_shell_surface(struct weston_wm_window *window,
 		&wm->server->compositor->shell_interface;
 	struct weston_output *output;
 	struct weston_wm_window *parent;
+	int flags = 0;
 
 	weston_wm_window_read_properties(window);
 
@@ -2305,6 +2469,13 @@ xserver_map_shell_surface(struct weston_wm_window *window,
 	if (!shell_interface->get_primary_view)
 		return;
 
+	if (window->surface->configure) {
+		weston_log("warning, unexpected in %s: "
+			   "surface's configure hook is already set.\n",
+			   __func__);
+		return;
+	}
+
 	window->shsurf = 
 		shell_interface->create_shell_surface(shell_interface->shell,
 						      window->surface,
@@ -2314,6 +2485,8 @@ xserver_map_shell_surface(struct weston_wm_window *window,
 
 	if (window->name)
 		shell_interface->set_title(window->shsurf, window->name);
+	if (window->pid > 0)
+		shell_interface->set_pid(window->shsurf, window->pid);
 
 	if (window->fullscreen) {
 		window->saved_width = window->width;
@@ -2334,11 +2507,22 @@ xserver_map_shell_surface(struct weston_wm_window *window,
 					      WL_SHELL_SURFACE_TRANSIENT_INACTIVE);
 	} else if (window->transient_for && window->transient_for->surface) {
 		parent = window->transient_for;
+		if (weston_wm_window_type_inactive(window))
+			flags = WL_SHELL_SURFACE_TRANSIENT_INACTIVE;
 		shell_interface->set_transient(window->shsurf,
 					       parent->surface,
 					       window->x - parent->x,
-					       window->y - parent->y, 0);
+					       window->y - parent->y, flags);
+	} else if (weston_wm_window_is_maximized(window)) {
+		shell_interface->set_maximized(window->shsurf);
 	} else {
-		shell_interface->set_toplevel(window->shsurf);
+		if (weston_wm_window_type_inactive(window)) {
+			shell_interface->set_xwayland(window->shsurf,
+							window->x,
+							window->y,
+							WL_SHELL_SURFACE_TRANSIENT_INACTIVE);
+		} else {
+			shell_interface->set_toplevel(window->shsurf);
+		}
 	}
 }
